#include "xmlparser.h"
#include <qfile.h>
#include <qxml.h>

XmlParser::Keyboard* XmlParser::parse( const QString& fileName )
{
  QFile file( fileName );
  if ( ! file.exists() ) {
    return 0;
  }
  
  _keyboard = new Keyboard();

  QXmlInputSource source( file );
  QXmlSimpleReader reader;
  reader.setContentHandler( this );
  reader.parse( source );
  return _keyboard;
}

bool XmlParser::startElement( const QString&, const QString&, const QString& qName,
                              const QXmlAttributes& attributes )
{
  if ( qName == "hardware-keyboard" ) {
  }
  else if ( qName == "key-config" ) {
    _keyConfig = new KeyConfig();
    _keyboard->addKeyConfig( _keyConfig );
  }
  else if ( qName == "key" ) {
    Key* key = new Key( attributes.value("keycode").toInt(), attributes.value("state").toInt() );
    _keyConfig->addKey(key);
  }
  else if ( qName == "replacement" ) {
    _keyConfig->setReplacement( attributes.value("text") );
  }
  else {
    qFatal("Unknow tag %s", qName.latin1());
  }
  return true;
}

XmlParser::Keyboard::Keyboard()
{
  _items.setAutoDelete( true );
}

XmlParser::KeyConfig::KeyConfig()
{
  _items.setAutoDelete( true );
}

void XmlParser::Keyboard::addKeyConfig( KeyConfig* keyConf )
{
  _items.append( keyConf );
}

void XmlParser::KeyConfig::addKey( Key* key )
{
  _items.append( key );
}

void XmlParser::KeyConfig::setReplacement( const QString& txt ) 
{
  _replacement = txt;
}

XmlParser::Key::Key( int keycode, int state ) 
  : _keycode( keycode ), _state( state )
{
}

int XmlParser::Key::key() const
{
  return _keycode;
}

int XmlParser::Key::state() const
{
  return _state;
}

XmlParser::KeyList& XmlParser::KeyConfig::items()
{
  return _items;
}

QString XmlParser::KeyConfig::replacement() const 
{
  return _replacement;
}

XmlParser::KeyConfigList& XmlParser::Keyboard::items()
{
  return _items;
}


