#include "configwidget.h"
#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qevent.h>
#include <qpe/config.h>
#include <qpe/qcopenvelope_qws.h>
#include "keyconfig.h"
#include <qhbox.h>
#include <qvgroupbox.h>
#include <qtabwidget.h>
#include "keyconfigset.h"
#include <qpe/global.h>
#include <qfile.h>
#include <qtextstream.h>
#include "xmlparser.h"
#include <qframe.h>

ConfigWidget::ConfigWidget( QWidget* parent, const char* name = 0 ) :QTabWidget( parent, name )
{
  setCaption("Custom Input Settings");
  
  QCopEnvelope* env = new QCopEnvelope( "CustomInput/Private", "disable()");
  delete env;
  
  // -------------------------------------------------- general
  QWidget* tab1 = new QWidget( this );
  addTab( tab1, tr("General") );
  QVBoxLayout* lay1 = new QVBoxLayout( tab1, 6 );
  
  // Letters line input:
  QHBoxLayout* lay2 = new QHBoxLayout( lay1, 6 );
  
  QLabel* label = new QLabel(tr("Virtual keyboard:"), tab1);
  lay2->addWidget( label );
  _edit = new QLineEdit( tab1 );
  lay2->addWidget( _edit );
  
  // Font Size line:
  QHBoxLayout* lay3 =new QHBoxLayout( lay1, 6 );

  label = new QLabel( tr("Virtual keyboard font size: "), tab1 );
  lay3->addWidget( label );
 
  _fontSize = new QSpinBox( 6, 48, 1, tab1);
  _fontSize->setValue(12);
  lay3->addWidget( _fontSize );
  lay3->addStretch( 1 );

  QFrame* line = new QFrame( tab1 );
  line->setFrameStyle( QFrame::HLine + QFrame::Plain );
  lay1->addWidget( line );

  QHBoxLayout* lay5 = new QHBoxLayout( lay1 );
  label = new QLabel( tr("<qt>Maximum delay between keypair on physical keyboard to be grouped for translation</qt>"), tab1);
  lay5->addWidget( label );
  
  _delay = new QSpinBox( 100, 10000, 100, tab1 );
  _delay->setSuffix("ms");
  lay5->addWidget( _delay );  


  lay1->addStretch( 1 );

  // -------------------------------------------------- Physical Keyboard
  QWidget* tab2 = new QWidget( this );
  addTab( tab2, tr("Physical Keyboard") );
  QVBoxLayout* lay4 = new QVBoxLayout( tab2, 6 );
  
  
  // -------------------------------------------------- Read configuration
  Config config( "CustomInput" );
  config.setGroup( "CustomInput" );
  _edit->setText( config.readEntry("symbols", "\\" ) );
  _fontSize->setValue( config.readNumEntry("fontSize", 12 ) );
  _delay->setValue( config.readNumEntry("delay", 1000 ) );
  
  XmlParser parser;
  QString fileName = Global::applicationFileName("custominput", "hardware-keyboard.xml");
  XmlParser::Keyboard* keyboard = parser.parse( fileName );
  if ( keyboard ) {
    _hardwareKeyboard = new KeyConfigSet( keyboard->items(), tab2 );
  }
  else {
    _hardwareKeyboard = new KeyConfigSet( tab2 );
  }
  lay4->addWidget( _hardwareKeyboard );
}

void ConfigWidget::closeEvent( QCloseEvent* e ) 
{
  // Virtual Keyboard
  Config config( "CustomInput" );
  config.setGroup( "CustomInput" );
  config.writeEntry("symbols", _edit->text() );
  config.writeEntry("fontSize", _fontSize->value() );
  config.writeEntry("delay", _delay->value() );
  
  // Physical Keyboard
  QString fileName = Global::applicationFileName("custominput", "hardware-keyboard.xml");
  QFile file( fileName );
  if ( ! file.open( IO_WriteOnly ) ) {
    qDebug("Ups!");
  }
  else {
    QTextStream stream( &file );
    QString str = _hardwareKeyboard->xml();
    stream << str;
  }
  file.close();

  QCopEnvelope* env = new QCopEnvelope( "CustomInput/Private", "reread()");
  delete env;

  env = new QCopEnvelope( "CustomInput/Private", "enable()");
  delete env;

  QWidget::closeEvent( e );
}
