#include "inputbutton.h"
#include <qevent.h>

InputButton::InputButton( QWidget* parent, const char* name ) : QPushButton( parent, name ), _changed( false )
{
  QPushButton::setText( tr("Select Key") );
  connect( this, SIGNAL( clicked() ), this, SLOT( takeFocus() ) );
}

InputButton::InputButton( int keycode, int state, QWidget* parent, const char* name )
  : QPushButton( parent, name ), _changed( false )
{
  setText( keycode, state );
  connect( this, SIGNAL( clicked() ), this, SLOT( takeFocus() ) );
}

void InputButton::takeFocus() 
{
  setFocus();
}

void InputButton::keyPressEvent( QKeyEvent* event ) 
{
  setText( event->key(), event->state() );
}

void InputButton::setText( int key, int state )
{
  QString txt;
  if ( state & ControlButton ) {
    txt = "Sh+";
  }
  else if ( state & ShiftButton ) {
    txt = "Sh+";
  }
  
  bool ok = true;
  switch ( key ) {

    // Small letters
  case Key_Q: txt +="q"; break;    
  case Key_1: txt +="1"; break;    

  case Key_W: txt +="w"; break;    
  case Key_2: txt +="2"; break;    

  case Key_E: txt +="e"; break;    
  case Key_3: txt +="3"; break;    

  case Key_R: txt +="r"; break;    
  case Key_4: txt +="4"; break;    

  case Key_T: txt +="t"; break;    
  case Key_5: txt +="5"; break;    

  case Key_Y: txt +="y"; break;    
  case Key_6: txt +="6"; break;    

  case Key_U: txt +="u"; break;    
  case Key_7: txt +="7"; break;    

  case Key_I: txt +="i"; break;    
  case Key_8: txt +="8"; break;    

  case Key_O: txt +="o"; break;    
  case Key_9: txt +="9"; break;    

  case Key_P: txt +="p"; break;    
  case Key_0: txt +="0"; break;    

  case Key_A: txt +="a"; break;    
  case Key_Exclam: txt +="!"; break;    

  case Key_S: txt +="s"; break;    
  case Key_At: txt +="@"; break;    

  case Key_D: txt +="d"; break;    
  case Key_NumberSign: txt +="#"; break;    

  case Key_F: txt +="f"; break;    
  case Key_Dollar: txt +="$"; break;    

  case Key_G: txt +="g"; break;    
  case Key_Percent: txt +="%"; break;    

  case Key_H: txt +="h"; break;    
  case Key_Underscore: txt +="_"; break;    

  case Key_J: txt +="j"; break;    
  case Key_Ampersand: txt +="&"; break;    

  case Key_K: txt +="k"; break;    
  case Key_Asterisk: txt +="*"; break;    

  case Key_L: txt +="l"; break;    
  case Key_ParenLeft: txt +="("; break;    

  case Key_Backspace: txt +="backspace"; break;    
  case Key_Delete: txt += "delete"; break;
  case Key_BraceLeft: txt = "Shift + Delete"; break;
    
  case Key_Z: txt +="z"; break;    
  case Key_X: txt +="x"; break;
  case Key_C: txt +="c"; break;
  case Key_V: txt +="v"; break;    

  case Key_B: txt +="b"; break;    
  case Key_Minus: txt +="-"; break;    

  case Key_N: txt +="n"; break;    
  case Key_Plus: txt +="+"; break;    

  case Key_M: txt +="m"; break;    
  case Key_Equal: txt +="="; break;    

  case Key_Comma:  txt +=","; break;    
  case Key_Semicolon: txt =";"; break;
  case Key_ParenRight: 
    if ( state & ShiftButton ) {
      txt ="Fn+Sh+,"; break;
    }
    else {
      txt =")"; break;
    }
    
    
  case Key_Slash: txt +="/"; break;    
  case Key_Question: txt = "?"; break;
  // fn + / is numlock

  case Key_Space: txt +="space"; break;
  case Key_QuoteLeft: txt ="Fn+Sh+space"; break;

  case Key_Apostrophe: txt +="'"; break;    
  case Key_QuoteDbl: txt = "\""; break;
  case Key_AsciiTilde: txt +="~"; break;
    
  case Key_Period: txt +="."; break;    
  case Key_Colon: txt += ":"; break;
  case  Key_Less: txt +="<"; break;
    
  case Key_Return: txt +="return"; break;    
  case Key_Greater: txt +=">"; break;
  
  default: ok = false;
  }
  
  if ( ok ) {
    QPushButton::setText( txt );
    _changed = true;
    _key = key;
    _state = state;
  }
}

bool InputButton::changed() const 
{
  return _changed;
}

int InputButton::key() const 
{
  return _key;
}

int InputButton::state() const 
{
  return _state;
}
