#include "inputbutton.h"
#include <qlayout.h>
#include <qlineedit.h>
#include "keyconfig.h"
#include <qpushbutton.h>
#include <qlabel.h>
#include <qframe.h>

KeyConfig::KeyConfig( QWidget* parent, const char* name )
  :QWidget( parent, name )
{
  setup();
  addButton();
}

KeyConfig::KeyConfig( const XmlParser::KeyList& keys, QWidget* parent, const char* name = 0 )
  :QWidget( parent, name )
{
  setup();
  for ( XmlParser::KeyListIterator it( keys ); *it; ++it ) {
    InputButton* but = new InputButton( (*it)->key(), (*it)->state(), this );
    _layout->insertWidget( _items.count()+1, but );
    _items.append( but );
  }
}


void KeyConfig::setup() 
{
  QVBoxLayout* lay1 = new QVBoxLayout( this, 6 );
  
  _line = new QFrame( this );
  _line->setFrameStyle( QFrame::HLine + QFrame::Plain );
  lay1->addWidget( _line );
  
  _layout = new QHBoxLayout( lay1, 6 );

  QLabel* label = new QLabel( tr("Key(s):"), this );
  _layout->addWidget( label );

  QPushButton* more = new QPushButton(">>", this );
  _layout->addWidget( more );
  connect( more, SIGNAL( clicked() ), this, SLOT(addButton()));
  
  QHBoxLayout* lay2 = new QHBoxLayout( lay1 );
  label = new QLabel( tr("Replacement:"), this );
  lay2->addWidget( label );
  
  _edit = new QLineEdit( this );
  lay2->addWidget( _edit );

  QHBoxLayout* lay3 = new QHBoxLayout( lay1 );
  QPushButton* del = new QPushButton( tr("delete"), this );
  lay3->addStretch(1);
  lay3->addWidget( del );
  connect( del, SIGNAL( clicked() ), this, SLOT(del()));
}

void KeyConfig::addButton()
{
  InputButton* but = new InputButton( this );
  _layout->insertWidget( _items.count()+1, but );
  but->show();
  _items.append( but );
}

QString KeyConfig::xml() const
{
  QString res = "\t<key-config>\n";
  bool anything = false;
  for ( QListIterator<InputButton> it(_items); *it; ++it ) {
    if ( (*it)->changed() ) {
      res += QString("\t\t<key keycode=\"") + QString::number((*it)->key()) + "\" state=\""+QString::number((*it)->state()) + "\"/>\n";
      anything = true;
    }
  }
  
  if ( !_edit->text().isEmpty() ) {
    res += "\t\t<replacement text=\"" + _edit->text().local8Bit() + "\"/>\n";
  }
  res += "\t</key-config>\n";
  
  anything = anything && !_edit->text().isEmpty();
  if ( anything )
    return res;
  else
    return QString::null;
}

void KeyConfig::del() 
{
  emit deleteMe(this);
}

void KeyConfig::setReplacement( const QString& replacement )
{
  _edit->setText( replacement );
}
