#include "keyconfigset.h"
#include "keyconfig.h"
#include <qscrollview.h>
#include <qlayout.h>
#include "xmlparser.h"
#include <qscrollbar.h>
#include <qtimer.h>

KeyConfigSet::KeyConfigSet( QWidget* parent, const char* name )
  :QWidget( parent, name )
{
  setup();
  add();
}

KeyConfigSet::KeyConfigSet( const XmlParser::KeyConfigList& keyList, QWidget* parent, const char* name = 0 )
  :QWidget( parent, name )
{
  setup();
  for ( XmlParser::KeyConfigListIterator it( keyList ); *it; ++it ) {
    add( (*it)->replacement(), (*it)->items() );
  }
}

void KeyConfigSet::setup()
{
  QVBoxLayout* lay1 = new QVBoxLayout( this );
  _view = new QScrollView( this );
  lay1->addWidget( _view, 1 );

  _top = new QWidget( _view->viewport() );
  _view->addChild( _top );
  _view->setResizePolicy( QScrollView::AutoOneFit );
  _layout = new QVBoxLayout( _top );

  _layout->addStretch(1);

  QPushButton* but = new QPushButton( tr("Add Key Combination"), this );
  lay1->addWidget( but );  
  connect( but, SIGNAL( clicked() ), this, SLOT(add()) );
}


void KeyConfigSet::add() 
{
  KeyConfig* config = new KeyConfig( _top );
  add( config );
}

void KeyConfigSet::add( const QString& replacement, const XmlParser::KeyList& keys ) 
{
  KeyConfig* config = new KeyConfig( keys, _top );
  config->setReplacement( replacement );
  add( config );
}

void KeyConfigSet::add( KeyConfig* config ) 
{
  connect( config, SIGNAL( deleteMe( KeyConfig* ) ), this, SLOT( del( KeyConfig* ) ) );
  _layout->insertWidget( _items.count(), config );
  _items.append( config );
  config->show();
  QTimer::singleShot( 0, this, SLOT( forceRepaint() ) );
}

// Due to what seems like a bug in Qt, the scrollview is not updated
// correctly when new elements  are added. Thefore this function is called
// to force repainting. It is however necesary to insert this into tre
// event loop (the singleShot timer), for this to work.
void KeyConfigSet::forceRepaint()
{
  _view->updateContents( 0,0, _view->contentsWidth(), _view->contentsHeight());
  _view->update();
  QScrollBar* bar = _view->horizontalScrollBar();
  if ( bar )
    bar->update();
  bar = _view->verticalScrollBar();
  if ( bar )
    bar->update(); 
}
void KeyConfigSet::del( KeyConfig* conf ) 
{
  _items.remove( conf );
  delete conf;
}

  
QString KeyConfigSet::xml() const
{
  QString res;
  res = "<hardware-keyboard>\n";
  for ( QListIterator<KeyConfig> it( _items ); *it; ++it ) {
    res += (*it)->xml();
  }
  res += "</hardware-keyboard>";
  return res;
}

