#include "custominput.h"
#include "inputwidget.h"
#include <qpixmap.h>
#include <qapplication.h>
#include <qwindowsystem_qws.h>
#include "keyboardhandler.h"
#include <qcopchannel_qws.h>

/* XPM */
static const char * pb_xpm[]={
"28 7 3 1",
"# c #303030",
"@ c #00FF00",
"  c None",
" #############   ##### ## # ",
" #@@@#@@@#@@@#   #@@@#      ",
" #@@@#@@@#@@@#   #####  ##  ",
" ###############            ",
" #@@@@@#@@@#@@@#            ",
" #@@@@@#@@@#@@@#            ",
" ###############            "};

CustomInput::CustomInput() : _inputWidget(0), _icn(0)
{
  _keyboardHandler =  new KeyboardHandler( this, "_keyboardHandler" );
  qwsServer->setKeyboardFilter( _keyboardHandler ); 
  _chanel = new QCopChannel( "CustomInput/Private", this );
  connect( _chanel, SIGNAL( received(const QCString &, const QByteArray &)),
           _keyboardHandler, SLOT(message( const QCString& )) );
  connect( _chanel, SIGNAL( received(const QCString &, const QByteArray &)),
           this, SLOT(message( const QCString& )) );
}

CustomInput::~CustomInput()
{
  delete _inputWidget;
  delete _icn;
  _inputWidget = 0;
  _icn = 0;
}

void CustomInput::message( const QCString& message ) 
{
  if ( message == "enable()" ) {
    qwsServer->setKeyboardFilter( _keyboardHandler );
  }
  else if ( message == "disable()" ) {
    qwsServer->setKeyboardFilter( 0 );
  }
}

QWidget* CustomInput::inputMethod( QWidget *parent, Qt::WFlags f ) 
{
  if ( ! _inputWidget ) {
    _inputWidget = new InputWidget( parent, "_inputWidget", f );
    connect( _chanel, SIGNAL( received(const QCString &, const QByteArray &)),
             _inputWidget, SLOT(message( const QCString & )) );
  }

  return _inputWidget;
}

void CustomInput::resetState() 
{
}

QPixmap* CustomInput::icon()
{    
  if ( !_icn )
	_icn = new QPixmap( (const char **)pb_xpm );
    return _icn;
}

QString CustomInput::name()
{
  return qApp->translate( "InputMethods", "Custom" );
}

void CustomInput::onKeyPress( QObject *receiver, const char *slot )
{   
  if ( _inputWidget )
    QObject::connect( _inputWidget, SIGNAL(key(ushort,ushort,ushort,bool,bool)), receiver, slot );
    QObject::connect( _keyboardHandler, SIGNAL(key(ushort,ushort,ushort,bool,bool)), receiver, slot );
}


QRESULT CustomInput::queryInterface( const QUuid &uuid, QUnknownInterface **iface )
{
  *iface = 0;
  if ( uuid == IID_QUnknown )
    *iface = this;
  else if ( uuid == IID_InputMethod )
    *iface = this;
  
  if ( *iface )
    (*iface)->addRef();
  return QS_OK;
}

Q_EXPORT_INTERFACE()
{
  Q_CREATE_INSTANCE( CustomInput )
}

