#include "keyboardhandler.h"
#include "xmlparser.h"
#include <qpe/global.h>
#include <qlabel.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qpe/config.h>

KeyboardHandler::KeyboardHandler( QObject* parent, const char* name ) : QObject( parent, name ), _input(0)
{
  reread();
  _time.start();
}

bool KeyboardHandler::filter(int /*unicode*/, int keycode, int modifiers, 
                             bool isPress, bool /*autoRepeat*/) 
{
  if ( !_active )
    return false;
  
  bool handled = false;
  if ( isPress ) {
    _input[_index] = Key( keycode, modifiers );
    
    for ( QValueList<Binding>::Iterator it = _bindings.begin(); 
          it != _bindings.end(); ++it ) {
      QValueList<Key> keys = (*it)._keys;
      QString replacement = (*it)._replacement;
      bool match = true;
      for ( unsigned int i = 0; i < keys.count(); ++i ) {
        if ( _input[(_index-keys.count()+1+i+_length)%_length] != keys[i] ) {
          match = false;
          break;
        }
      }
      if ( match ) {
        if ( _time.elapsed() < _delay ) {
          for ( unsigned int i = 0; i< keys.count()-1; i++) {
            // delete the previous characters
            emit key( 0, Qt::Key_BackSpace, 0, true, false );
            emit key( 0, Qt::Key_BackSpace, 0, false, false );
          }
          for ( unsigned int i = 0; i < replacement.length(); ++i ) {
            emit key( replacement[i].unicode(), 0, 0, true, false );
            emit key( replacement[i].unicode(), 0, 0, false, false );
          }
          handled = true; // Dont insert this characters
        }
        break;
      }
    }
    _index = (_index+1)%_length;
    _time.restart();
  }
  return handled;
}

void KeyboardHandler::message( const QCString& message )
{
  if ( message == "reread()" )
    reread();
}

void KeyboardHandler::reread()
{
  _bindings.clear();

  XmlParser parser;
  QString fileName = Global::applicationFileName("custominput", "hardware-keyboard.xml");
  XmlParser::Keyboard* keyboard = parser.parse( fileName );
  if ( keyboard && keyboard->items().count() > 0 ) {    
    _active = true;
    _length = 0;
    for ( XmlParser::KeyConfigListIterator it( keyboard->items() ); *it; ++it ) {
      XmlParser::KeyConfig* config = *it;
      Binding binding;
      binding.setReplacement( config->replacement() );
      int count = 0;
      for ( XmlParser::KeyListIterator it2( config->items() ); *it2; ++it2 ) 
      {
        binding.addKey( Key( (*it2)->key(), (*it2)->state() ) );
        count++;
      }
      _length = QMAX( _length, count );
      _bindings.append( binding );
    }
    delete[] _input;
    _input = new Key[_length];
    _index = 0;
    for ( int i = 0; i< _length; ++i ) {
      _input[i] = Key(0,0);
    }
  }
  else {
    _active = false;
  }
  delete keyboard;

  Config config( "CustomInput" );
  config.setGroup( "CustomInput" );
  _delay = config.readNumEntry("delay", 1000 );

}

